#!/usr/bin/env python
"""
_LumiBased_t_

Lumi based splitting tests, using the DataStructs classes.
See WMCore/WMBS/JobSplitting/ for the WMBS (SQL database) version.
"""

from builtins import next, range

import unittest

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Fileset import Fileset
from WMCore.DataStructs.Job import Job
from WMCore.DataStructs.Subscription import Subscription
from WMCore.DataStructs.Workflow import Workflow
from WMCore.DataStructs.Run import Run

from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.Services.UUIDLib import makeUUID

class LumiBasedTest(unittest.TestCase):
    """
    _LumiBasedTest_

    Test event based job splitting.
    """


    def setUp(self):
        """
        _setUp_

        Create two subscriptions: One that contains a single file and one that
        contains multiple files.
        """

        self.testWorkflow = Workflow()
        self.performanceParams = {'timePerEvent' : 12,
                                  'memoryRequirement' : 2300,
                                  'sizePerEvent' : 400}

        return

    def tearDown(self):
        """
        _tearDown_

        Nothing to do...
        """
        pass


    def createSubscription(self, nFiles, lumisPerFile, twoSites = False):
        """
        _createSubscription_

        Create a subscription for testing
        """

        baseName = makeUUID()

        testFileset = Fileset(name = baseName)
        for i in range(nFiles):
            newFile = File(lfn = '%s_%i' % (baseName, i), size = 1000,
                           events = 100)
            lumis = []
            for lumi in range(lumisPerFile):
                lumis.append((i * 100) + lumi)
            newFile.addRun(Run(i, *lumis))
            newFile.setLocation('blenheim')
            testFileset.addFile(newFile)
        if twoSites:
            for i in range(nFiles):
                newFile = File(lfn = '%s_%i_2' % (baseName, i), size = 1000,
                               events = 100)
                lumis = []
                for lumi in range(lumisPerFile):
                    lumis.append(5 + 10 * (i * 100) + lumi) #lumis should be different
                newFile.addRun(Run(i, *lumis))
                newFile.setLocation('malpaquet')
                testFileset.addFile(newFile)


        testSubscription  = Subscription(fileset = testFileset,
                                         workflow = self.testWorkflow,
                                         split_algo = "LumiBased",
                                         type = "Processing")

        return testSubscription

    def testA_FileSplitting(self):
        """
        _FileSplitting_

        Test that things work if we split files between jobs
        """
        splitter = SplitterFactory()

        oneSetSubscription = self.createSubscription(nFiles = 10, lumisPerFile = 1)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = oneSetSubscription)


        jobGroups = jobFactory(lumis_per_job = 3,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertTrue(len(job['input_files']), 1)




        twoLumiFiles = self.createSubscription(nFiles = 5, lumisPerFile = 2)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = twoLumiFiles)
        jobGroups = jobFactory(lumis_per_job = 1,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job['input_files']), 1)



        wholeLumiFiles = self.createSubscription(nFiles = 5, lumisPerFile = 3)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = wholeLumiFiles)
        jobGroups = jobFactory(lumis_per_job = 2,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        # 10 because we split on run boundaries
        self.assertEqual(len(jobGroups[0].jobs), 10)
        jobList = jobGroups[0].jobs
        for job in jobList:
            # Have should have one file, half two
            self.assertTrue(len(job['input_files']) in [1,2])


        mask0 = jobList[0]['mask'].getRunAndLumis()
        self.assertEqual(mask0, {0: [[0, 1]]})
        mask1 = jobList[1]['mask'].getRunAndLumis()
        self.assertEqual(mask1, {0: [[2, 2]]})
        mask2 = jobList[2]['mask'].getRunAndLumis()
        self.assertEqual(mask2, {1: [[100, 101]]})
        mask3 = jobList[3]['mask'].getRunAndLumis()
        self.assertEqual(mask3, {1: [[102, 102]]})

        self.assertEqual(jobList[0]['mask'].getRunAndLumis(), {0: [[0, 1]]})

        # Do it with multiple sites
        twoSiteSubscription = self.createSubscription(nFiles = 5, lumisPerFile = 2, twoSites = True)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = twoSiteSubscription)
        jobGroups = jobFactory(lumis_per_job = 1,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams)
        self.assertEqual(len(jobGroups), 2)
        self.assertEqual(len(jobGroups[0].jobs), 10)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job['input_files']), 1)



    def testB_NoRunNoFileSplitting(self):
        """
        _NoRunNoFileSplitting_

        Test the splitting algorithm in the odder fringe
        cases that might be required.
        """
        splitter = SplitterFactory()
        testSubscription = self.createSubscription(nFiles = 5, lumisPerFile = 5, twoSites = False)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = testSubscription)

        jobGroups = jobFactory(lumis_per_job = 3,
                               halt_job_on_file_boundaries = False,
                               splitOnRun = False,
                               performance = self.performanceParams)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 9)

        # The first job should have three lumis from one run
        # The second three lumis from two different runs
        self.assertEqual(jobs[0]['mask'].getRunAndLumis(), {0: [[0, 2]]})
        job1runLumi = jobs[1]['mask'].getRunAndLumis()
        self.assertEqual(job1runLumi[0][0][0] + 1, job1runLumi[0][0][1])  # Run 0, startLumi+1 == endLumi
        self.assertEqual(job1runLumi[1][0][0], job1runLumi[1][0][1])  # Run 1, startLumi == endLumi

        # Assert that this works differently with file splitting on and run splitting on
        testSubscription = self.createSubscription(nFiles = 5, lumisPerFile = 5, twoSites = False)
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = testSubscription)
        jobGroups = jobFactory(lumis_per_job = 3,
                               halt_job_on_file_boundaries = True,
                               splitOnRun = True,
                               performance = self.performanceParams)
        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 10)

        # In this case it should slice things up so that each job only has one run
        # in it.
        self.assertEqual(jobs[0]['mask'].getRunAndLumis(), {0: [[0, 2]]})
        self.assertEqual(jobs[1]['mask'].getRunAndLumis(), {0: [[3, 4]]})
        return


    def testC_LumiCorrections(self):
        """
        _LumiCorrections_

        Test the splitting algorithm can handle lumis which
        cross multiple files.
        """
        splitter = SplitterFactory()
        testSubscription = self.createSubscription(nFiles = 2, lumisPerFile = 2, twoSites = False)
        files = testSubscription.getFileset().getFiles()
        self.assertEqual(len(files), 2)
        for runObj in files[0]['runs']:
            if runObj.run != 0:
                continue
            runObj.appendLumi(42)
        for runObj in files[1]['runs']:
            if runObj.run != 1:
                continue
            runObj.run = 0
            runObj.appendLumi(42)
        files[1]['locations'] = set(['blenheim'])

        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = testSubscription)

        jobGroups = jobFactory(lumis_per_job = 3,
                               halt_job_on_file_boundaries = False,
                               splitOnRun = False,
                               performance = self.performanceParams,
                               applyLumiCorrection = True
                              )

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 2)

        self.assertEqual(len(jobs[0]['input_files']), 2)
        self.assertEqual(len(jobs[1]['input_files']), 1)
        self.assertEqual(jobs[0]['mask'].getRunAndLumis(), {0: [[0, 1], [42, 42]]})
        self.assertEqual(jobs[1]['mask'].getRunAndLumis(), {0: [[100, 101]]})

        #Test that we are not removing all the lumis from the jobs anymore
        removedLumi = self.createSubscription(nFiles = 4, lumisPerFile = 1)
        #Setting the lumi of job 0 to value 100, as the one of job one
        runObj = next(iter(removedLumi.getFileset().getFiles()[0]['runs']))
        runObj.run = 1
        runObj[0] = 100
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = removedLumi)
        jobGroups = jobFactory(lumis_per_job = 1,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams,
                               applyLumiCorrection = True)
        # we need to end up with 3 jobs and one job with two input files
        jobs = jobGroups[0].jobs

        self.assertEqual(len(jobs), 3)
        self.assertEqual(len(jobs[0]['input_files']), 2)
        self.assertEqual(len(jobs[1]['input_files']), 1)
        self.assertEqual(len(jobs[2]['input_files']), 1)
        self.assertEqual(jobs[0]['mask'].getRunAndLumis(), {1: [[100, 100]]})
        self.assertEqual(jobs[1]['mask'].getRunAndLumis(), {2: [[200, 200]]})
        self.assertEqual(jobs[2]['mask'].getRunAndLumis(), {3: [[300, 300]]})


        #Check that if the last two jobs have the same duplicated lumi you do not get an error
        testSubscription = self.createSubscription(nFiles = 2, lumisPerFile = 2,
                                           twoSites = False)
        files = testSubscription.getFileset().getFiles()
        # Now modifying and adding the same duplicated lumis in the Nth and Nth-1 jobs
        for runObj in files[0]['runs']:
            if runObj.run != 0:
                continue
            runObj.appendLumi(42)
        for runObj in files[1]['runs']:
            runObj.run = 0
            runObj.lumis = [42]
        files[1]['locations'] = set(['blenheim'])
        jobFactory = splitter(package = "WMCore.DataStructs",
                              subscription = testSubscription)
        jobGroups = jobFactory(events_per_job = 50,
                               halt_job_on_file_boundaries = True,
                               performance = self.performanceParams,
                               applyLumiCorrection = True)

        self.assertEqual(len(jobGroups), 1)
        jobs = jobGroups[0].jobs
        self.assertEqual(len(jobs), 3)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_GetChildIDsByID_

MySQL implementation of File.GetChildIDsByID

Return a list of ids which are children for a file(s) with a given id(s).
"""




from WMCore.Database.DBFormatter import DBFormatter

class GetChildIDsByID(DBFormatter):
    sql = """select distinct child from wmbs_file_parent where parent = :parent"""

    def getBinds(self, ids=None):
        binds = []
        childIDs = self.dbi.makelist(ids)
        for id in childIDs:
            binds.append({'parent': id})
        return binds

    def format(self, result):
        out = set()
        for r in result:
            if isinstance(r, int):
                # deal with crappy mysql implementation
                out.add(int(r))
            else:
                for f in r.fetchall():
                    out.add(int(f[0]))
            r.close()
        return list(out)

    def execute(self, ids=None, conn = None, transaction = False):
        binds = self.getBinds(ids)
        result = self.dbi.processData(self.sql, binds,
                         conn = conn, transaction = transaction)
        return self.format(result)



#!/usr/bin/env python

"""
__Performance__

Base class for Database Performance Tests

This class is abstract, serving as a superclass for all
DB Performance Testcases

"""
from __future__ import print_function

import logging, time

class Performance:
    """
    __Performance__

    Base class for Database Performance Tests

    This class is abstract, serving as a superclass for all
    DB Performance Testcases


    """

    def setUp(self):
        """
        Common setUp for all DB Performance tests

        """

        #Setting up logger
        logging.basicConfig(level=logging.DEBUG,
                   format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                   datefmt='%m-%d %H:%M',
                   filename='%s.log' % __file__.replace('.py',''),
                   filemode='w')
        self.logger = logging.getLogger('BasePerformanceTest')

    def tearDown(self):
        """
        Common tearDown for all DB Performance tests

        """
        pass

    def formatExecInput(self, input):
        """
        Method that format the string received as an argument
        for the execute method of each DAO class.

        """
        fmtstring = ""
        for i in input.keys():
            if type(input[i]) == type('string'):
                fmtstring = "%s %s='%s'," % (fmtstring, i, input[i])
            else:
                fmtstring = "%s %s=%s," % (fmtstring, i, input[i])
        fmtstring = fmtstring.strip()
        fmtstring = fmtstring.rstrip(",")
        return fmtstring

    def perfTest(self, dao, action, **input):
        """
        Method that executes a dao class operation and measures its
        execution time.

        """
        actionstring = action

        action = dao(classname=action)
        string = self.formatExecInput(input=input)
        string = "action.execute(%s)" % string
        self.logger.debug('the final string: %s' % string)
        diffTotal = 0

        #Performance testing block START
        startTime = time.time()
        #Place execute method of the specific classname here
        #string = compile(string)
        eval(string)
        endTime = time.time()
        diffTime = endTime - startTime
        #Performance testing block END
        diffTotal = diffTotal + diffTime
        if self.verbose == 'True':
            print(string)

        assert diffTotal <= self.threshold, actionstring+' DAO class - '+ \
                       'Operation too slow ( elapsed time:'+ str(diffTotal)+ \
                        ', threshold:'+str(self.threshold)+' )'
        return diffTotal



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unit tests for the WMWorkloadTools module
"""
from __future__ import print_function, division

import unittest

from Utils.PythonVersion import PY3
from WMCore.WMSpec.WMWorkloadTools import checkMemCore, checkEventStreams, checkTimePerEvent


class WMWorkloadToolsTest(unittest.TestCase):

    def setUp(self):
        """
        Setup unit tests for this module
        """
        pass

    def tearDown(self):
        """
        Cleanup after the unit tests
        """
        pass

    def testCheckMemCore(self):
        """
        Tests for the function 'checkMemCore'
        """
        # input that fails validation
        self.assertFalse(checkMemCore(-1))
        self.assertFalse(checkMemCore(0))
        self.assertFalse(checkMemCore(0.5, minValue=1))
        self.assertFalse(checkMemCore(0, minValue=1))
        # input that passes validation
        self.assertTrue(checkMemCore(1))
        self.assertTrue(checkMemCore(0, minValue=0))
        self.assertTrue(checkMemCore(0.5, minValue=0))
        self.assertTrue(checkMemCore(1.0, minValue=1))
        self.assertTrue(checkMemCore(1, minValue=1))
        self.assertTrue(checkMemCore({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        self.assertTrue(checkMemCore({"Task1": 1, "Task2": 2, "Task3": 3.5}, minValue=1))
        self.assertTrue(checkMemCore({"Task1": 1, "Task2": 2, "Task3": 3.5}, minValue=0))
        # tests with different behaviour depending on the python version
        if PY3:
            self.assertFalse(checkMemCore("-1"))
            self.assertFalse(checkMemCore([1]))
            self.assertFalse(checkMemCore((1, 2)))
        else:
            self.assertTrue(checkMemCore("-1"))
            self.assertTrue(checkMemCore([1]))
            self.assertTrue(checkMemCore((1, 2)))

    def testCheckEventStreams(self):
        """
        Tests for the function 'checkEventStreams'
        """
        # input that fails validation
        self.assertFalse(checkEventStreams(-1))
        # input that passes validation
        self.assertTrue(checkEventStreams(1))
        self.assertTrue(checkEventStreams(0))
        self.assertTrue(checkEventStreams(0.5))
        self.assertTrue(checkEventStreams(1.0))
        self.assertTrue(checkEventStreams(1))
        self.assertTrue(checkEventStreams({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        self.assertTrue(checkEventStreams({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        self.assertTrue(checkEventStreams({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        # tests with different behaviour depending on the python version
        if PY3:
            self.assertFalse(checkEventStreams("-1"))
            self.assertFalse(checkEventStreams([1]))
            self.assertFalse(checkEventStreams((1, 2)))
        else:
            self.assertTrue(checkEventStreams("-1"))
            self.assertTrue(checkEventStreams([1]))
            self.assertTrue(checkEventStreams((1, 2)))


    def testCheckTimePerEvent(self):
        """
        Tests for the function 'checkTimePerEvent'
        """
        # input that fails validation
        self.assertFalse(checkTimePerEvent(-1))
        # input that passes validation
        self.assertTrue(checkTimePerEvent(1))
        self.assertTrue(checkTimePerEvent(0))
        self.assertTrue(checkTimePerEvent(0.5))
        self.assertTrue(checkTimePerEvent(1.0))
        self.assertTrue(checkTimePerEvent(1))
        self.assertTrue(checkTimePerEvent({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        self.assertTrue(checkTimePerEvent({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        self.assertTrue(checkTimePerEvent({"Task1": 1, "Task2": 2, "Task3": 3.5}))
        # tests with different behaviour depending on the python version
        if PY3:
            self.assertFalse(checkTimePerEvent("-1"))
            self.assertFalse(checkTimePerEvent([1]))
            self.assertFalse(checkTimePerEvent((1, 2)))
        else:
            self.assertTrue(checkTimePerEvent("-1"))
            self.assertTrue(checkTimePerEvent([1]))
            self.assertTrue(checkTimePerEvent((1, 2)))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""
Unit tests for Singleton pattern
"""




import unittest
from WMCore.Algorithms.Singleton import Singleton

class SingletonTest(unittest.TestCase):
    """
    _SingletonTest_
    """

    def testIDs(self):
        """
        Make sure there really are two objects with the same underlying object
        """
        s1 = Singleton()
        s2 = Singleton()

        self.assertEqual(s1.singletonId(), s2.singletonId())
        self.assertNotEqual(id(s1), id(s2))



if __name__ == "__main__":
    unittest.main()



"""
_PileupFetcher_t_

The purpose of this test is to track what is happening in the PileupFetcher module.
The test instantiates a MC worklow and calls the PileupFetcher on its generation task.
"""
from __future__ import print_function

from future.utils import viewitems

import os
import unittest
from json import JSONDecoder

from Utils.PythonVersion import PY3

import WMCore.WMSpec.WMStep as WMStep
import WMCore.WMSpec.WMTask as WMTask
from WMCore.Database.CMSCouch import CouchServer, Document
from WMCore.Services.DBS.DBS3Reader import DBS3Reader
from WMCore.Services.Rucio.Rucio import Rucio
from WMCore.WMRuntime.SandboxCreator import SandboxCreator
from WMCore.WMSpec.StdSpecs.TaskChain import TaskChainWorkloadFactory
from WMCore.WMSpec.Steps.Fetchers.PileupFetcher import PileupFetcher
from WMCore.WMSpec.WMWorkloadTools import parsePileupConfig
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInitCouchApp import TestInitCouchApp


class PileupFetcherTest(EmulatedUnitTestCase):
    def setUp(self):
        """
        Initialize the database and couch.

        """
        super(PileupFetcherTest, self).setUp()

        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setupCouch("pileupfetcher_t", "ConfigCache")
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        couchServer = CouchServer(os.environ["COUCHURL"])
        self.configDatabase = couchServer.connectDatabase("pileupfetcher_t")
        self.testDir = self.testInit.generateWorkDir()
        self.rucioAcct = "wmcore_transferor"
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        """
        Clear out the database.
        """
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        super(PileupFetcherTest, self).tearDown()  # Left here in case it's needed by any of the sub-classes

    def injectGenerationConfig(self):
        """
        _injectGenerationConfig_

        Inject a generation config for the MC workflow.
        """
        config = Document()
        config["info"] = None
        config["config"] = None
        config["md5hash"] = "eb1c38cf50e14cf9fc31278a5c8e580f"
        config["pset_hash"] = "7c856ad35f9f544839d8525ca10259a7"
        config["owner"] = {"group": "cmsdataops", "user": "sfoulkes"}
        config["pset_tweak_details"] = None
        config["pset_tweak_details"] = \
            {"process": {"outputModules_": ["OutputA"],
                         "OutputA": {"dataset": {"filterName": "OutputAFilter",
                                                 "dataTier": "GEN-SIM-RAW"}}}}
        result = self.configDatabase.commitOne(config)
        return result[0]["id"]

    def _queryAndCompareWithDBS(self, pileupDict, pileupConfig, dbsUrl):
        """
        pileupDict is a Python dictionary containing particular pileup
        configuration information. Query DBS on given dataset contained
        now in both input pileupConfig as well as in the pileupDict
        and compare values.
        """
        self.assertItemsEqual(list(pileupDict), list(pileupConfig))
        reader = DBS3Reader(dbsUrl)
        rucioObj = Rucio(self.rucioAcct)

        # now query DBS and compare the blocks and files from DBS
        # against those returned by the PileupFetcher
        for pileupType, datasets in viewitems(pileupConfig):
            # this is from the pileup configuration produced by PileupFetcher
            blockDict = pileupDict[pileupType]

            for dataset in datasets:
                dbsBlocks = reader.listFileBlocks(dataset=dataset)
                rucioBlocksLocation = rucioObj.getPileupLockedAndAvailable(dataset,
                                                                           account=self.rucioAcct)

                # first, validate the number of blocks and their names
                self.assertItemsEqual(list(blockDict), dbsBlocks)
                self.assertItemsEqual(list(blockDict), list(rucioBlocksLocation))
                # now validate the block location between Rucio and PileupFetcher
                for block, blockLocation in viewitems(blockDict):
                    self.assertItemsEqual(blockLocation['PhEDExNodeNames'], rucioBlocksLocation[block])

                    # finally, validate the files
                    fileList = []
                    # now get list of files in the block
                    dbsFiles = reader.listFilesInBlock(block)
                    for dbsFile in dbsFiles:
                        fileList.append(dbsFile["LogicalFileName"])
                    self.assertItemsEqual(blockDict[block]["FileList"], fileList)

    def _queryPileUpConfigFile(self, pileupConfig, task, taskPath):
        """
        Query and compare contents of the the pileup JSON
        configuration files. Iterate over tasks's steps as
        it happens in the PileupFetcher.
        """
        for step in task.steps().nodeIterator():
            helper = WMStep.WMStepHelper(step)
            # returns e.g. instance of CMSSWHelper
            if hasattr(helper.data, "pileup"):
                decoder = JSONDecoder()

                stepPath = "%s/%s" % (taskPath, helper.name())
                pileupPath = "%s/%s" % (stepPath, "pileupconf.json")
                try:
                    with open(pileupPath) as fObj:
                        pileupDict = decoder.decode(fObj.read())
                except IOError:
                    m = "Could not read pileup JSON configuration file: '%s'" % pileupPath
                    self.fail(m)
                self._queryAndCompareWithDBS(pileupDict, pileupConfig, helper.data.dbsUrl)

    def testPileupFetcherOnMC(self):
        pileupMcArgs = TaskChainWorkloadFactory.getTestArguments()
        pileupMcArgs['Task1']["MCPileup"] = "/Cosmics/ComissioningHI-PromptReco-v1/RECO"
        pileupMcArgs['Task1']["DataPileup"] = "/HighPileUp/Run2011A-v1/RAW"
        pileupMcArgs['Task1']["ConfigCacheID"] = self.injectGenerationConfig()
        pileupMcArgs["CouchDBName"] = "pileupfetcher_t"
        pileupMcArgs["CouchURL"] = os.environ["COUCHURL"]

        factory = TaskChainWorkloadFactory()
        testWorkload = factory.factoryWorkloadConstruction("TestWorkload", pileupMcArgs)

        # now that the workload was created and args validated, we can add this PileupConfig
        pileupMcArgs["PileupConfig"] = parsePileupConfig(pileupMcArgs['Task1']["MCPileup"],
                                                         pileupMcArgs['Task1']["DataPileup"])
        # Since this is test of the fetcher - The loading from WMBS isn't
        # really necessary because the fetching happens before the workflow
        # is inserted into WMBS: feed the workload instance directly into fetcher:
        fetcher = PileupFetcher()
        creator = SandboxCreator()
        pathBase = "%s/%s" % (self.testDir, testWorkload.name())
        for topLevelTask in testWorkload.taskIterator():
            for taskNode in topLevelTask.nodeIterator():
                # this is how the call to PileupFetcher is happening
                # from the SandboxCreator test
                task = WMTask.WMTaskHelper(taskNode)
                taskPath = "%s/WMSandbox/%s" % (pathBase, task.name())
                fetcher.setWorkingDirectory(taskPath)
                # create Sandbox for the fetcher ...
                creator._makePathonPackage(taskPath)
                fetcher(task)
                self._queryPileUpConfigFile(pileupMcArgs["PileupConfig"], task, taskPath)


if __name__ == "__main__":
    unittest.main()



"""
_Create_DBSBuffer_

Implementation of Create_DBSBuffer for Oracle.
"""

import threading

from WMCore.Database.DBCreator import DBCreator

class Create(DBCreator):

    def __init__(self, logger = None, dbi = None, params = None):
        """
        _init_

        Call the base class's constructor and create all necessary tables,
        constraints and inserts.
        """
        myThread = threading.currentThread()
        DBCreator.__init__(self, myThread.logger, myThread.dbi)

        #
        # Tables, functions, procedures and sequences
        #
        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_dataset (
                 id                INTEGER        NOT NULL,
                 path              VARCHAR2(500)  NOT NULL,
                 processing_ver    INTEGER        NOT NULL,
                 acquisition_era   VARCHAR2(255),
                 valid_status      VARCHAR2(20),
                 global_tag        VARCHAR2(255),
                 parent            VARCHAR2(500),
                 prep_id           VARCHAR(255),
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_dat UNIQUE (path)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_dataset_subscription (
                 id                 INTEGER      NOT NULL,
                 dataset_id         INTEGER      NOT NULL,
                 site               VARCHAR(100) NOT NULL,
                 custodial          INTEGER      DEFAULT 0,
                 priority           VARCHAR(10)  DEFAULT 'Low',
                 subscribed         INTEGER      DEFAULT 0,
                 delete_blocks      INTEGER,
                 dataset_lifetime   INTEGER      DEFAULT 0 NOT NULL,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_dat_sub UNIQUE (dataset_id, site, custodial, priority)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_algo (
                 id             INTEGER        NOT NULL,
                 app_name       VARCHAR2(100),
                 app_ver        VARCHAR2(100),
                 app_fam        VARCHAR2(100),
                 pset_hash      VARCHAR2(700),
                 config_content CLOB,
                 in_dbs         INTEGER,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_alg UNIQUE (app_name, app_ver, app_fam, pset_hash)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_algo_dataset_assoc (
                 id         INTEGER,
                 algo_id    INTEGER NOT NULL,
                 dataset_id INTEGER NOT NULL,
                 in_dbs     INTEGER DEFAULT 0,
                 PRIMARY KEY (id)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_workflow (
                 id                           INTEGER,
                 name                         VARCHAR2(700),
                 task                         VARCHAR2(700),
                 block_close_max_wait_time    INTEGER,
                 block_close_max_files        INTEGER,
                 block_close_max_events       INTEGER,
                 block_close_max_size         INTEGER,
                 completed                    INTEGER        DEFAULT 0,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_wor UNIQUE (name, task)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_file (
                 id                    INTEGER,
                 lfn                   VARCHAR2(500) NOT NULL,
                 filesize              INTEGER,
                 events                INTEGER,
                 cksum                 INTEGER,
                 dataset_algo          INTEGER       NOT NULL,
                 status                VARCHAR2(20),
                 in_phedex             INTEGER       DEFAULT 0,
                 block_id              INTEGER,
                 workflow              INTEGER,
                 LastModificationDate  INTEGER,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_fil UNIQUE (lfn)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_file_parent (
                 child  NUMBER(11) NOT NULL,
                 parent NUMBER(11) NOT NULL,
                 CONSTRAINT pk_dbs_fil_par PRIMARY KEY (child, parent)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_file_runlumi_map (
                 filename  INTEGER NOT NULL,
                 run       INTEGER NOT NULL,
                 lumi      INTEGER NOT NULL,
                 num_events    INTEGER
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_location (
                 id       INTEGER,
                 pnn  VARCHAR2(255) NOT NULL,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_loc UNIQUE (pnn)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_file_location (
                 filename   INTEGER NOT NULL,
                 location   INTEGER NOT NULL,
                 CONSTRAINT pk_dbs_fil_loc PRIMARY KEY (filename, location)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_block (
                 id          INTEGER,
                 dataset_id  INTEGER      NOT NULL,
                 blockname   VARCHAR(250) NOT NULL,
                 location    INTEGER      NOT NULL,
                 create_time INTEGER,
                 status      VARCHAR(20),
                 deleted     INTEGER      DEFAULT 0,
                 rule_id     VARCHAR(40)  DEFAULT '0' NOT NULL,
                 PRIMARY KEY (id),
                 CONSTRAINT uq_dbs_blo UNIQUE (blockname, location)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_checksum_type (
                 id    INTEGER,
                 type  VARCHAR(255),
                 PRIMARY KEY (id)
               )"""

        self.create[len(self.create)] = \
            """CREATE TABLE dbsbuffer_file_checksums (
                 fileid  INTEGER,
                 typeid  INTEGER,
                 cksum   VARCHAR2(100),
                 CONSTRAINT pk_dbs_fil_che PRIMARY KEY (fileid, typeid)
               )"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_dataset_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_dataset_sub_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_algo_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_algdset_assoc_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_file_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_location_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_block_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_checksum_type_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""

        self.create[len(self.create)] = \
            """CREATE SEQUENCE dbsbuffer_workflow_seq
                 START WITH 1
                 INCREMENT BY 1
                 NOMAXVALUE"""


        self.create[len(self.create)] = \
            """CREATE TRIGGER dbsbuffer_file_trg
                 BEFORE INSERT ON dbsbuffer_file
                 FOR EACH ROW
                  BEGIN
                    SELECT dbsbuffer_file_seq.nextval INTO :new.id FROM dual;
                  END;"""

        #
        # Indexes
        #
        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_che_1 ON dbsbuffer_file_checksums (typeid)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_che_2 ON dbsbuffer_file_checksums (fileid)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_dat_sub_1 ON dbsbuffer_dataset_subscription (dataset_id)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_par_1 ON dbsbuffer_file_parent (child)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_par_2 ON dbsbuffer_file_parent (parent)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_blo_1 ON dbsbuffer_block (location)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_blo_2 ON dbsbuffer_block (dataset_id)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_alg_ass_1 ON dbsbuffer_algo_dataset_assoc (dataset_id)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_alg_ass_2 ON dbsbuffer_algo_dataset_assoc (algo_id)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_run_1 ON dbsbuffer_file_runlumi_map (filename)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_loc_1 ON dbsbuffer_file_location (location)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_loc_2 ON dbsbuffer_file_location (filename)"""

        self.indexes[len(self.indexes)] = \
            """CREATE INDEX idx_dbs_fil_1 ON dbsbuffer_file (workflow)"""

        #
        # Constraints
        #
        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_checksums
                 ADD CONSTRAINT fk_file_checksums_typeid
                 FOREIGN KEY (typeid)
                 REFERENCES dbsbuffer_checksum_type(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_checksums
                 ADD CONSTRAINT fk_file_checksums_fileid
                 FOREIGN KEY (fileid)
                 REFERENCES dbsbuffer_file(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_dataset_subscription
                 ADD CONSTRAINT fk_dsetsubscription_datasetid
                 FOREIGN KEY (dataset_id)
                 REFERENCES dbsbuffer_dataset(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_parent
                 ADD CONSTRAINT fk_file_parent_child
                 FOREIGN KEY (child)
                 REFERENCES dbsbuffer_file(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_parent
                 ADD CONSTRAINT fk_file_parent_parent
                 FOREIGN KEY (parent)
                 REFERENCES dbsbuffer_file(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_block
                 ADD CONSTRAINT fk_block_location
                 FOREIGN KEY (location)
                 REFERENCES dbsbuffer_location(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_block
                 ADD CONSTRAINT fk_block_dataset_id
                 FOREIGN KEY (dataset_id)
                 REFERENCES dbsbuffer_dataset(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_algo_dataset_assoc
                 ADD CONSTRAINT fk_algodset_assoc_dataset_id
                 FOREIGN KEY (dataset_id)
                 REFERENCES dbsbuffer_dataset(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_algo_dataset_assoc
                 ADD CONSTRAINT fk_algodset_assoc_algo_id
                 FOREIGN KEY (algo_id)
                 REFERENCES dbsbuffer_algo(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_runlumi_map
                 ADD CONSTRAINT fk_file_runlumi_filename
                 FOREIGN KEY (filename)
                 REFERENCES dbsbuffer_file(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_location
                 ADD CONSTRAINT fk_file_location_location
                 FOREIGN KEY (location)
                 REFERENCES dbsbuffer_location(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file_location
                 ADD CONSTRAINT fk_file_location_filename
                 FOREIGN KEY (filename)
                 REFERENCES dbsbuffer_file(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file
                 ADD CONSTRAINT fk_file_workflow
                 FOREIGN KEY (workflow)
                 REFERENCES dbsbuffer_workflow(id)
                 ON DELETE CASCADE"""

        self.constraints[len(self.constraints)] = \
            """ALTER TABLE dbsbuffer_file
                 ADD CONSTRAINT fk_file_dataset_algo
                 FOREIGN KEY (dataset_algo)
                 REFERENCES dbsbuffer_algo_dataset_assoc(id)
                 ON DELETE CASCADE"""


        checksumTypes = ['cksum', 'adler32', 'md5']
        for i in checksumTypes:
            checksumTypeQuery = """INSERT INTO dbsbuffer_checksum_type
                                   (id, type)
                                   VALUES (dbsbuffer_checksum_type_seq.nextval, '%s')
                                   """ % (i)
            self.inserts["wmbs_checksum_type_%s" % (i)] = checksumTypeQuery



"""
Unit tests for MSManager.py module

Author: Valentin Kuznetsov <vkuznet [AT] gmail [DOT] com>
"""

from __future__ import division, print_function

import unittest

from Utils.PythonVersion import PY3

from WMCore.Agent.Configuration import Configuration
from WMCore.MicroService.MSCore.MSManager import MSManager


class MSManagerTest(unittest.TestCase):
    "Unit test for MSManager module"

    def setUp(self):
        "Setup MSManager for testing"
        config = Configuration()
        data = config.section_('data')
        data.reqmgr2Url = "http://localhost/reqmgr2"
        data.verbose = True
        data.interval = 600
        data.quotaUsage = 0.8
        data.quotaAccount = "DataOps"
        data.enableStatusTransition = True
        data.rucioAccount = "wma_test"
        data.rucioUrl = "http://cms-rucio-int.cern.ch"
        data.rucioAuthUrl = "https://cms-rucio-auth-int.cern.ch"
        data.phedexUrl = "https://cmsweb.cern.ch/phedex/datasvc/json/prod"
        data.dbsUrl = "https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader"
        data.smtpServer = "localhost"
        data.fromAddr = "noreply@cern.ch"
        data.toAddr = ["cms-comp-ops-workflow-team@cern.ch"]
        data.warningTransferThreshold = 100. * (1000 ** 4)  # 100 TB (terabyte)
        self.mgr = MSManager(data)

        data.services = ['monitor']
        self.mgr_monit = MSManager(data)

        data.services = ['transferor']
        data.limitRequestsPerCycle = 50
        data.enableDataTransfer = True
        self.mgr_trans = MSManager(data)

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        "Tear down MSManager"
        self.mgr.stop()
        self.mgr_trans.stop()
        self.mgr_monit.stop()

    def test_services(self):
        "test MSManager services"
        # check self.mgrt object attributes
        self.assertItemsEqual(self.mgr.services, [])
        self.assertEqual(hasattr(self.mgr, 'msTransferor'), False)
        self.assertEqual(hasattr(self.mgr, 'transfThread'), False)
        self.assertEqual(hasattr(self.mgr, 'msMonitor'), False)
        self.assertEqual(hasattr(self.mgr, 'monitThread'), False)

        # check self.mgr_trans object attributes
        self.assertEqual('monitor' in self.mgr_trans.services, False)
        self.assertEqual('transferor' in self.mgr_trans.services, True)
        self.assertEqual(hasattr(self.mgr_trans, 'msTransferor'), True)
        self.assertEqual(hasattr(self.mgr_trans, 'transfThread'), True)
        self.assertEqual(hasattr(self.mgr_trans, 'msMonitor'), False)
        self.assertEqual(hasattr(self.mgr_trans, 'monitThread'), False)

        # check self.mgr_monit object attributes
        self.assertEqual('monitor' in self.mgr_monit.services, True)
        self.assertEqual('transferor' in self.mgr_monit.services, False)
        self.assertEqual(hasattr(self.mgr_monit, 'msTransferor'), False)
        self.assertEqual(hasattr(self.mgr_monit, 'transfThread'), False)
        self.assertEqual(hasattr(self.mgr_monit, 'msMonitor'), True)
        self.assertEqual(hasattr(self.mgr_monit, 'monitThread'), True)

        # test a few configuration parameters as well
        self.assertEqual(self.mgr_trans.msConfig.get("limitRequestsPerCycle"), 50)
        self.assertFalse("limitRequestsPerCycle" in self.mgr_monit.msConfig)
        self.assertTrue(self.mgr_trans.msConfig.get("enableStatusTransition"))
        self.assertTrue("enableStatusTransition" in self.mgr_monit.msConfig)
        self.assertTrue(self.mgr_trans.msConfig.get("enableDataTransfer"))
        self.assertFalse("enableDataTransfer" in self.mgr_monit.msConfig)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_ListJobsBySub_

Oracle implementation of Monitoring.ListJobsBySub
"""




from WMCore.WMBS.MySQL.Monitoring.ListJobsBySub import ListJobsBySub \
 as ListJobsBySubMySQL

class ListJobsBySub(ListJobsBySubMySQL):
    pass



#!/usr/bin/env python
"""
_New_

Oracle implementation of JobGroup.New
"""

__all__ = []



import time

from WMCore.WMBS.MySQL.JobGroup.New import New as NewJobGroupMySQL

class New(NewJobGroupMySQL):
    sql = """INSERT INTO wmbs_jobgroup (id, subscription, guid, output,
             last_update) VALUES (wmbs_jobgroup_SEQ.nextval, :subscription,
             :guid, :output, %d)""" % time.time()



from __future__ import print_function, division

# TODO: need to get algorithm proper way
ALGO_DICT = {"FileBased": {"files_per_job": ''},
             "TwoFileBased": {"two_files_per_job": ''},
             "LumiBased": {"lumis_per_job": '',
                           "halt_job_on_file_boundaries": True},
             "EventAwareLumiBased": {"events_per_job": '',
                                     "job_time_limit": '',
                                     "halt_job_on_file_boundaries": True},
             "EventBased": {"events_per_job": '',
                            "events_per_lumi": ''},
             "Harvest": {"periodic_harvest_interval": ''},
             "ParentlessMergeBySize": {"min_merge_size": '',
                                       "max_merge_size": '',
                                       "max_merge_events": '',
                                       "max_wait_time": ''},
             "WMBSMergeBySize": {"min_merge_size": '',
                                 "max_merge_size": '',
                                 "max_merge_events": '',
                                 "max_wait_time": ''},
             "MergeBySize": {"min_merge_size": '',
                             "max_merge_size": '',
                             "max_merge_events": '',
                             "max_wait_time": ''}
             }

ALGO_LIST_BY_TYPES = {"Processing": ["LumiBased", "EventAwareLumiBased",
                                     "EventBased", "FileBased"],
                      "Production": ["EventBased"],
                      "Skim": ["FileBased", "TwoFileBased"],
                      "Harvesting": ["Harvest"],
                      "Merge": ["ParentlessMergeBySize",
                                "WMBSMergeBySize",
                                "MergeBySize"],
                      "Cleanup": ["FileBased"],
                      "LogCollect": ["FileBased"]
                      }

EDITABLE_SPLITTING_PARAM_CONFIG = {"algo_params": ALGO_DICT,
                                   "algo_list_by_types": ALGO_LIST_BY_TYPES
                                   }



from builtins import map, range, object, str, bytes

from WMCore.WebTools.RESTModel import RESTModel, restexpose
from cherrypy import HTTPError

DUMMY_ROLE = "dummy"
DUMMY_GROUP = "dummies"
DUMMY_SITE = "dummyHome"

class DummyDAO1(object):
    """
    A DAO that has no arguments and does nothing but return 123
    """
    def execute(self):
        return 123

class DummyDAO2(object):
    """
    A DAO that takes a single argument
    """
    def execute(self, num):
        return {'num': num}

class DummyDAO3(object):
    """
    A DAO with keyword arguments
    TODO: use this
    """
    def execute(self, num, thing=None):
        return {'num': num, 'thing': thing}

class DummyDAOFac(object):
    """
    Something that replicates a Factory that loads our dummy DAO classes
    """
    def __call__(self, classname='DummyDAO1'):
        dao = None
        if classname == 'DummyDAO1':
            dao = DummyDAO1()
        elif classname == 'DummyDAO2':
            dao = DummyDAO2()
        elif classname == 'DummyDAO3':
            dao = DummyDAO3()
        return dao

class DummyRESTModel(RESTModel):
    def __init__(self, config):
        '''
        Initialise the RESTModel and add some methods to it.
        '''
        RESTModel.__init__(self, config)
        self.defaultExpires = config.default_expires

        self.methods = {'GET':{
                               'ping': {'default_data':1234,
                                        'call':self.ping,
                                        'version': 1,
                                        'args': [],
                                        'expires': 3600,
                                        'validation': []},
                               #'list1': {'call':self.list1,
                               #         'version': 1}
                               },
                        'POST':{
                               'echo': {'call':self.echo,
                                        'version': 1,
                                        'args': ['message'],
                                        'validation': []},
                               }
                         }

        self._addMethod('GET', 'gen', self.gen)

        self._addMethod('GET', 'list', self.list, args=['input_int', 'input_str'],
                              validation=[self.val_0,
                                          self.val_1,
                                          self.val_2,
                                          self.val_3,
                                          self.val_4 ],
                              version=2)

        self._addMethod('GET', 'list1', self.list1)
        self._addMethod('GET', 'list2', self.list2, args=['num0', 'num1', 'num2'])
        self._addMethod('GET', 'list3', self.list3, args=['a', 'b'])
        self._addMethod('POST', 'list3', self.list3, args=['a', 'b'])
        self._addMethod('PUT', 'list1', self.list1, secured = True,
                        security_params = {'role':DUMMY_ROLE,
                                           'group': DUMMY_GROUP,
                                           'site':DUMMY_SITE})

        # a will take list of numbers. i.e. a[1,2,3]
        self._addMethod('GET', 'listTypeArgs', self.listTypeArgs, args=['aList'],
                       validation = [self.listTypeValidate])

        self.daofactory = DummyDAOFac()
        self._addDAO('GET', 'data1', 'DummyDAO1', [])
        self._addDAO('GET', 'data2', 'DummyDAO2', ['num'])
        self._addDAO('GET', 'data3', 'DummyDAO3', ['num', 'thing'])

    @restexpose
    def ping(self):
        """
        Return a simple message
        """
        return 'ping'

    @restexpose
    def echo(self, *args, **kwargs):
        """
        Echo back the arguments sent to the call. If sanitise needed to be called
        explicitly (e.g. method not added via _addMethod) method signature of callee
        should be (*args, **kwargs).
        """
        input_data = self._sanitise_input(args, kwargs, 'echo')
        return input_data

    def gen(self):
        """Generator method which produce generator dicts"""
        data = ({'idx':i} for i in range(10))
        return data

    def list(self, input_int, input_str):
        return {'input_int':input_int, 'input_str':input_str}

    def list1(self):
        """ test no argument case, return 'No argument' string """
        return 'No argument'

    def list2(self, num0, num1, num2):
        """ test multiple argment string return the dictionary of key: value pair of
            the arguments """
        return {'num0': num0, 'num1': num1, 'num2': num2}

    def list3(self, *args, **kwargs):
        """ test sanitise without any validation specified """
        return kwargs

    def listTypeArgs(self, aList):
        """ test whether it handles ?aList=1&aList=2 types of query """
        return aList

    def listTypeValidate(self, request_input):
        if not isinstance(request_input["aList"], list):
            request_input["aList"] = [int(request_input["aList"])]
        else:
            request_input["aList"] = list(map(int, request_input["aList"]))
        return request_input

    def val_0(self, request_input):
        # checks whether request_input is right number
        if len(request_input) != 2:
            raise HTTPError(400, 'val_0 failed: request_input length is not 2 -- (%s)' % len(request_input))
        return request_input

    def val_1(self, request_input):
        # Convert the request_input data to an int (will be a string), ignore if it
        # fails as the next validation will kill that, and it makes the unit test
        # trickier...
        try:
            request_input['input_int'] = int(request_input['input_int'])
        except:
            pass
        # Checks its first request_input contains a int
        try:
            assert isinstance(request_input['input_int'], type(123))
        except AssertionError:
            raise AssertionError('val_1 failed: %s not int' % type(request_input['input_int']))
        return request_input

    def val_2(self, request_input):
        # Checks its second request_input is a string
        try:
            assert isinstance(request_input['input_str'], (str, bytes))
        except AssertionError:
            raise HTTPError(400, 'val_2 failed: %s not string or unicode' % type(request_input['input_str']))
        return request_input

    def val_3(self, request_input):
        # Checks the int is 123
        try:
            assert request_input['input_int'] == 123
        except AssertionError:
            raise HTTPError(400, 'val_3 failed: %s != 123' % request_input['input_int'])

        return request_input

    def val_4(self, request_input):
        # Checks the str is 'abc'
        try:
            assert request_input['input_str'] == 'abc'
        except AssertionError:
            raise HTTPError(400, 'val_4 failed: %s != "abc"' % request_input['input_str'])

        return request_input



#!/usr/bin/env python
# encoding: utf-8
from builtins import range
import logging
import unittest
import os
from WMCore.WMLogging import CouchHandler
from WMCore.Database.CMSCouch import CouchServer

class WMLoggingTest(unittest.TestCase):
    def setUp(self):
        # Make an instance of the server
        self.server = CouchServer(os.getenv("COUCHURL", 'http://admin:password@localhost:5984'))
        testname = self.id().split('.')[-1]
        # Create a database, drop an existing one first
        self.dbname = 'cmscouch_unittest_%s' % testname.lower()

        if self.dbname in self.server.listDatabases():
            self.server.deleteDatabase(self.dbname)

        self.server.createDatabase(self.dbname)
        self.db = self.server.connectDatabase(self.dbname)

    def tearDown(self):
        # This used to test self._exc_info to only run on success. Broke in 2.7. Removed.
        self.server.deleteDatabase(self.dbname)

    def testLog(self):
        """
        Write ten log messages to the database at three different levels
        """
        my_logger = logging.getLogger('MyLogger')
        my_logger.setLevel(logging.DEBUG)
        handler = CouchHandler(self.server.url, self.dbname)
        formatter = logging.Formatter('%(message)s')
        handler.setFormatter(formatter)
        my_logger.addHandler(handler)

        for _ in range(10):
            my_logger.debug('This is probably all noise.')
            my_logger.info('Jackdaws love my big sphinx of quartz.')
            my_logger.error('HOLLY CRAP!')
        logs = self.db.allDocs()['rows']
        self.assertEqual(30, len(logs))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_AssociateWorkflowToFile_

Oracle implementation of DBS3Buffer.DBSBufferFiles.AssociateWorkflowToFile

Created on Oct 12, 2012

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.AssociateWorkflowToFile import \
     AssociateWorkflowToFile as MySQLAssociateWorkfloToFile

class AssociateWorkflowToFile(MySQLAssociateWorkfloToFile):
    pass



#!/usr/bin/env python


"""
Checks for finished subscriptions
Upon finding finished subscriptions, notifies WorkQueue and kills them
"""
from __future__ import print_function




import logging
import threading

from WMCore.Agent.Harness import Harness
#from WMCore.WMFactory import WMFactory

from WMCore.BossAir.StatusPoller import StatusPoller


class JobStatusLite(Harness):
    """
    Checks for finished subscriptions
    Upon finding finished subscriptions, notifies WorkQueue and kills them

    """

    def __init__(self, config):
        # call the base class
        Harness.__init__(self, config)

        self.config = config
        #self.config.JobStatusLite.pollInterval
        #self.config.JobStatusLite.queryInterval

        print("JobStatusLite.__init__")

    def preInitialization(self):
        """
        Sets up the worker thread

        """
        logging.info("JobStatusLite.preInitialization")

        # Add event loop to worker manager
        myThread = threading.currentThread()
        logging.info(str(myThread))


        logging.info("Setting poll interval to %s seconds" \
                      %str(self.config.JobStatusLite.pollInterval) )
        myThread.workerThreadManager.addWorker( \
                              StatusPoller(self.config), \
                              self.config.JobStatusLite.pollInterval \
                            )

        return



#!/usr/bin/env python
"""
CouchCollection_t.py

Created by Dave Evans on 2010-10-04.
Copyright (c) 2010 Fermilab. All rights reserved.
"""

from builtins import range
import unittest
import random

from WMQuality.TestInitCouchApp import TestInitCouchApp

from WMCore.ACDC.CouchCollection import CouchCollection
from WMCore.ACDC.CouchFileset import CouchFileset

from WMCore.DataStructs.File import File
from WMCore.GroupUser.User import makeUser
from WMCore.Services.UUIDLib import makeUUID


class CouchCollection_t(unittest.TestCase):
    """
    Unittest for Collection specialised for CouchDB backend
    """

    def setUp(self):
        """setup couch instance"""
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setupCouch("wmcore-acdc-couchcollection", "GroupUser",
                                 "ACDC")

        self.owner = makeUser("DMWM", "evansde77", self.testInit.couchUrl,
                              self.testInit.couchDbName)
        self.owner.connect()
        self.owner.create()

    def tearDown(self):
        """
        _tearDown_

        Clean up couch.
        """
        self.testInit.tearDownCouch()
        return

    def testCreatePopulateDrop(self):
        """
        _testCreatePopulateDrop_

        Test creating, populating and dropping a collection.
        """
        testCollectionA = CouchCollection(database=self.testInit.couchDbName,
                                          url=self.testInit.couchUrl,
                                          name="Thunderstruck")
        testCollectionB = CouchCollection(database=self.testInit.couchDbName,
                                          url=self.testInit.couchUrl,
                                          name="StruckThunder")
        testCollectionA.create()
        testCollectionB.create()

        # There should be nothing in couch.  Documents are only added for
        # filesets and files.

        testFilesA = []
        for i in range(5):
            testFile = File(lfn=makeUUID(), size=random.randint(1024, 4096),
                            events=random.randint(1024, 4096))
            testFilesA.append(testFile)
        testFilesB = []
        for i in range(10):
            testFile = File(lfn=makeUUID(), size=random.randint(1024, 4096),
                            events=random.randint(1024, 4096))
            testFilesB.append(testFile)

        testFilesetA = CouchFileset(database=self.testInit.couchDbName,
                                    url=self.testInit.couchUrl,
                                    name="TestFilesetA")
        testFilesetB = CouchFileset(database=self.testInit.couchDbName,
                                    url=self.testInit.couchUrl,
                                    name="TestFilesetB")
        testFilesetC = CouchFileset(database=self.testInit.couchDbName,
                                    url=self.testInit.couchUrl,
                                    name="TestFilesetC")
        testCollectionA.addFileset(testFilesetA)
        testCollectionB.addFileset(testFilesetB)
        testCollectionB.addFileset(testFilesetC)
        testFilesetA.add(testFilesA)
        testFilesetB.add(testFilesA)
        testFilesetC.add(testFilesA)
        testFilesetC.add(testFilesB)

        # Drop testCollectionA
        testCollectionA.drop()

        # Try to populate testFilesetA
        testCollectionC = CouchCollection(database=self.testInit.couchDbName,
                                          url=self.testInit.couchUrl,
                                          name="ThunderStruck")
        testCollectionC.populate()

        self.assertEqual(len(testCollectionC["filesets"]), 0,
                         "Error: There should be no filesets in this collect.")

        # Try to populate testFilesetB
        testCollectionD = CouchCollection(database=self.testInit.couchDbName,
                                          url=self.testInit.couchUrl,
                                          name="StruckThunder")
        testCollectionD.populate()

        for fileset in testCollectionD["filesets"]:
            testFiles = testFilesA
            if fileset["name"] == "TestFilesetC":
                testFiles.extend(testFilesB)

            self.assertEqual(len(testFiles), len(fileset.files),
                             "Error: Wrong number of files in fileset.")
            for testFile in testFiles:
                self.assertTrue(testFile["lfn"] in fileset.files,
                                "Error: File is missing.")
                self.assertEqual(testFile["events"],
                                 fileset.files[testFile["lfn"]]["events"],
                                 "Error: Wrong number of events.")
                self.assertEqual(testFile["size"],
                                 fileset.files[testFile["lfn"]]["size"],
                                 "Error: Wrong file size.")

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_LogDB_t_

LogDB tests
"""
from __future__ import print_function

import time
import logging
import unittest

from WMQuality.TestInitCouchApp import TestInitCouchApp
from WMCore.Services.LogDB.LogDB import LogDB
from WMCore.Services.LogDB.LogDBBackend import gen_hash
from WMCore.Services.LogDB.LogDBExceptions import LogDBError
from WMCore.Services.LogDB.LogDBReport import LogDBReport

class LogDBTest(unittest.TestCase):
    """
    _LogDBTest_

    """
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.
        """
        logging.basicConfig()
        self.logger = logging.getLogger('LogDBTest')
#        self.logger.setLevel(logging.DEBUG)

        self.schema = []
        self.couchApps = ["LogDB"]
        self.testInit = TestInitCouchApp('LogDBTest')
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = self.schema,
                                useDefault = False)
        dbName = 'logdb_t'
        self.testInit.setupCouch(dbName, *self.couchApps)
        url = "%s/%s" % (self.testInit.couchUrl, dbName)

        identifier = 'agentname'
        self.agent_inst = LogDB(url, identifier, logger=self.logger, thread_name="MainThread")
        self.agent_inst2 = LogDB(url, identifier, logger=self.logger, thread_name="Test")
        identifier = '/DC=org/DC=doegrids/OU=People/CN=First Last 123'
        self.user_inst = LogDB(url, identifier, logger=self.logger, thread_name="MainThread")
        self.report = LogDBReport(self.agent_inst)

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.tearDownCouch()

    def test_docid(self):
        "Test docid API"
        request = 'abs'
        mtype = self.agent_inst.backend.prefix('info')
        res = self.agent_inst.backend.docid(request, mtype)
        key = '--'.join([request, self.agent_inst.identifier, self.agent_inst.thread_name, mtype])
        khash = gen_hash(key)
        self.assertEqual(res, khash)

    def test_prefix(self):
        "Test prefix LogDBBackend API"

        self.assertEqual(self.agent_inst.agent, 1)
        mtype = self.agent_inst.backend.prefix('msg')
        self.assertEqual(mtype, 'agent-msg')

        self.assertEqual(self.user_inst.agent, 0)
        mtype = self.user_inst.backend.prefix('msg')
        self.assertEqual(mtype, 'msg')

    def test_apis(self):
        "Test LogDB APIs"
        request = 'abc'

        # if we post messages for the same request only last one should survive
        self.agent_inst.post(request, 'msg1')
#        time.sleep(1)
        self.agent_inst.post(request, 'msg2')
#        time.sleep(1)
        self.agent_inst.post(request, 'msg3')
#        time.sleep(1)
        self.agent_inst.post(request, 'msg4')
#        time.sleep(1)
        docs = self.agent_inst.get(request)
        self.assertEqual(len(docs), 1)
        self.assertEqual(docs[0]['msg'], 'msg4')

        # add message as user, user based messages will grow
        self.user_inst.post(request, 'doc1')
#        time.sleep(1)
        docs = self.user_inst.get(request)
        self.assertEqual(len(docs), 2) # we have msg4 and doc1
        self.user_inst.post(request, 'doc1') # we have msg2 and two doc1 messages
#        time.sleep(1)
        docs = self.user_inst.get(request)
        self.assertEqual(len(docs), 3)

    def test_cleanup(self):
        "Test clean-up LogDB API"
        request = 'abc'
        self.agent_inst.post(request, 'msg1', 'info')
        self.agent_inst.post(request, 'msg2', 'comment')
        self.agent_inst.post(request, 'msg3', 'warning')
        self.agent_inst.post(request, 'msg4', 'error')
        all_docs = self.agent_inst.get(request)
        self.agent_inst.backend.cleanup(thr=10) # look into past
        past_docs = self.agent_inst.get(request)
        self.assertEqual(len(all_docs), len(past_docs))
        self.agent_inst.backend.cleanup(thr=-10) # look into future
        docs = self.agent_inst.get(request)
        self.assertEqual(len(docs), 0)

    def test_get_all_requests(self):
        "Test get_all_requests LogDB API"
        request = 'abc'
        self.agent_inst.post(request, 'msg1', 'info')
        self.agent_inst.post(request, 'msg2', 'info')
        self.agent_inst.post(request, 'msg3', 'warning')
        self.agent_inst.post(request, 'msg4', 'error')
        sum_docs = self.agent_inst.get(request)
        self.assertEqual(len(sum_docs), 3) # since we have two info records
        request = 'abc2'
        self.agent_inst.post(request, 'msg1', 'info')
        all_docs = self.agent_inst.get_all_requests()
        self.assertEqual(len(all_docs), 2) # we only have two distinct request names

    def test_delete(self):
        "Test delete LogDB API"
        request = 'abc'
        self.agent_inst.post(request, 'msg1', 'info')
        self.agent_inst.delete(request)
        all_docs = self.agent_inst.get_all_requests()
        self.assertEqual(len(all_docs), 0)

    def test_get(self):
        "Test get LogDB API"
        request = 'abc'
        self.agent_inst2.post(request, 'msg1', 'info') # doc with different thread name
        self.agent_inst.post(request, 'msg1', 'info')
        self.agent_inst.post(request, 'msg2', 'info')
        self.agent_inst.post(request, 'msg3', 'warning')
        self.agent_inst.post(request, 'msg4', 'error')
        docs = self.agent_inst.get(request)
        self.assertEqual(len(docs), 4) # there should be 4 docs one for Test thread and 3 for MainThread
        docs = self.agent_inst2.get(request)
        self.assertEqual(len(docs), 4) # this should result the same as above.

        docs1 = self.agent_inst.get(request, 'info')
        docs2 = self.agent_inst.get(request, 'info')
        req1 = set([r['request'] for r in docs1])
        self.assertEqual(len(req1), 1)
        req2 = set([r['request'] for r in docs2])
        self.assertEqual(len(req2), 1)
        self.assertEqual(req1, req2)

    def test_thread_name(self):
        "Test thread name support by LogDB APIs"
        request = 'abc'
        self.agent_inst2.post(request, 'msg1', 'info') # doc with different thread name
        self.agent_inst.post(request, 'msg1', 'info')
        self.agent_inst.post(request, 'msg2', 'info')
        self.agent_inst.post(request, 'msg3', 'warning')
        self.agent_inst.post(request, 'msg4', 'error')
        maindocs = self.agent_inst.get(request)
        self.assertEqual(len(maindocs), 4) # since we have two info records in agent_inst
        thr_docs = self.agent_inst2.get(request)
        self.assertEqual(len(thr_docs), 4) # number of docs in different thread
        req1 = set([r['request'] for r in maindocs])
        self.assertEqual(len(req1), 1)
        req2 = set([r['request'] for r in thr_docs])
        self.assertEqual(len(req2), 1)
        self.assertEqual(req1, req2)
        worker1 = set([r['thr'] for r in maindocs])
        self.assertEqual(worker1, set(["Test", "MainThread"]))
        worker2 = set([r['thr'] for r in thr_docs])
        self.assertEqual(worker2, set(["Test", "MainThread"]))

        docs = set([r['identifier'] for r in maindocs])
        self.assertEqual(docs, set([self.agent_inst.identifier]))
        docs = set([r['identifier'] for r in thr_docs])
        self.assertEqual(docs, set([self.agent_inst.identifier]))

    def test_checks(self):
        "Tests LogDB check/mtype functionality"
        request = 'abc'
        res = self.agent_inst.post(request, 'msg', 'ingo') # should be silent, i.o. no Exceptions
        self.assertEqual('post-error', res)
        args = (request, 'msg', 'ingo') # wrong type
        self.assertRaises(LogDBError, self.agent_inst.backend.agent_update, *args)
        args = (request, 'ingo') # wrong type
        self.assertRaises(LogDBError, self.agent_inst.backend.check, *args)
        args = ('', 'msg', 'info') # empty request string
        self.assertRaises(LogDBError, self.agent_inst.backend.agent_update, *args)
        args = ('', 'info') # empty request string
        self.assertRaises(LogDBError, self.agent_inst.backend.check, *args)

    def test_report(self):
        "Test LogDBReport APIs"
        request = 'abc'
        self.agent_inst.post(request, 'msg1', 'info')
        time.sleep(1)
        self.agent_inst.post(request, 'msg2', 'comment')
        time.sleep(1)
        self.agent_inst.post(request, 'msg3', 'warning')
        time.sleep(1)
        self.agent_inst.post(request, 'msg4', 'error')
        time.sleep(1)
        self.report.to_stdout(request)
        out = self.report.to_txt(request)
        print("\n### txt report\n", out)
        out = self.report.to_html(request)
        print("\n### html report\n", out)

        docs = self.agent_inst.get(request)
        tdocs = self.report.orderby(docs, order='ts')
        messages = [d['msg'] for d in tdocs]
        self.assertEqual(messages, ['msg4', 'msg3', 'msg2', 'msg1'])

    def test_heartbeat(self):
        "Test heartbeat_report function"
        threadName = 'MainThread'
        failThread = 'FailThread'
        self.agent_inst.post(msg="Test heartbeat", mtype="info")
        time.sleep(1)
        self.agent_inst.post(msg="Test heartbeatError", mtype="error")
        report = self.agent_inst.heartbeat_report()
        self.assertEqual(report[threadName]['type'], 'agent-error')

        report = self.agent_inst.wmstats_down_components_report([threadName])
        self.assertEqual(report['down_components'], [threadName])

        self.agent_inst.post(msg="Test heartbeat", mtype="info")
        report = self.agent_inst.wmstats_down_components_report([threadName])
        self.assertEqual(report['down_components'], [])

        report = self.agent_inst.wmstats_down_components_report([threadName, failThread])
        self.assertEqual(report['down_components'], [failThread])

        self.agent_inst.post(msg="Test heartbeatError2", mtype="error")
        report = self.agent_inst.wmstats_down_components_report([threadName])
        self.assertEqual(report['down_components'], [threadName])

        self.agent_inst.delete(mtype="error", this_thread=True)
        report = self.agent_inst.wmstats_down_components_report([threadName])
        self.assertEqual(report['down_components'], [])

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_File_t_

Unit tests for the WMBS File class.
"""

from builtins import range
import threading
import unittest

import nose

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMQuality.TestInit import TestInit


class CursorLeakTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.  Also add some dummy locations.
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)

        myThread = threading.currentThread()
        daofactory = DAOFactory(package="WMCore.WMBS",
                                logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationAction = daofactory(classname="Locations.New")
        locationAction.execute(siteName="se1.cern.ch")
        locationAction.execute(siteName="se1.fnal.gov")

        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.clearDatabase()

    def testCursor(self):
        """
        _testCursor_

        test the cursor closing is really affected

        create 100 files with 5 parents and  loop 100 times.
        If the cursors are exhausted will crash.?

        TODO: improve for more effective testing.

        """

        raise nose.SkipTest
        fileList = []
        parentFile = None
        for i in range(100):
            testFile = File(lfn="/this/is/a/lfn%s" % i, size=1024, events=10,
                            checksums={"cksum": "1"})
            testFile.addRun(Run(1, *[i]))
            testFile.create()

            for j in range(5):
                parentFile = File(lfn="/this/is/a/lfnP%s" % j, size=1024,
                                  events=10, checksums={"cksum": "1"})
                parentFile.addRun(Run(1, *[j]))
                parentFile.create()
                testFile.addParent(parentFile['lfn'])

            fileList.append(testFile)

        for i in range(100):
            for file in fileList:
                file.loadData()
                file.getAncestors(level=2)
                file.getAncestors(level=2, type="lfn")

        return

    def testLotsOfAncestors(self):
        """
        _testLotsOfAncestors_

        Create a file with 15 parents with each parent having 100 parents to
        verify that the query to return grandparents works correctly.
        """
        raise nose.SkipTest
        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10,
                         checksums={"cksum": "1"}, locations="se1.fnal.gov")
        testFileA.create()

        for i in range(15):
            testParent = File(lfn=makeUUID(), size=1024, events=10,
                              checksums={"cksum": "1"}, locations="se1.fnal.gov")
            testParent.create()
            testFileA.addParent(testParent["lfn"])

            for i in range(100):
                testGParent = File(lfn=makeUUID(), size=1024, events=10,
                                   checksums={"cksum": "1"}, locations="se1.fnal.gov")
                testGParent.create()
                testParent.addParent(testGParent["lfn"])

        assert len(testFileA.getAncestors(level=2, type="lfn")) == 1500, \
            "ERROR: Incorrect grand parents returned"

        return


if __name__ == "__main__":
    unittest.main()



from WMCore.WMSpec.Persistency import PersistencyHelper
import unittest
from WMCore.WMSpec.WMStep import WMStep, makeWMStep


class PersistencyTest(unittest.TestCase):

    def testSplitUrl(self):
        helper = PersistencyHelper()
        url = 'https://cmsreqmgr.cern.ch/couchdb/mydb/doc/spec'
        base, dbname, doc = helper.splitCouchUrl(url)
        self.assertEqual(base, 'https://cmsreqmgr.cern.ch/couchdb')
        self.assertEqual(dbname, 'mydb')
        self.assertEqual(doc, 'doc/spec')

        url = 'http://localhost:5984/mydb/doc/spec'
        base, dbname, doc = helper.splitCouchUrl(url)
        self.assertEqual(base, 'http://localhost:5984')
        self.assertEqual(dbname, 'mydb')
        self.assertEqual(doc, 'doc/spec')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import print_function
import threading
import os
import sys

from DBSAPI.dbsApi import DbsApi
from dbsException import DbsException

from WMCore.WMInit import WMInit
from WMCore.Configuration import loadConfigurationFile

def connectToDB():
    """
    _connectToDB_

    Connect to the database specified in the WMAgent config.
    """
    if "WMAGENT_CONFIG" not in os.environ:
        print("Please set WMAGENT_CONFIG to point at your WMAgent configuration.")
        sys.exit(1)

    if not os.path.exists(os.environ["WMAGENT_CONFIG"]):
        print("Can't find config: %s" % os.environ["WMAGENT_CONFIG"])
        sys.exit(1)

    wmAgentConfig = loadConfigurationFile(os.environ["WMAGENT_CONFIG"])

    if not hasattr(wmAgentConfig, "CoreDatabase"):
        print("Your config is missing the CoreDatabase section.")

    socketLoc = getattr(wmAgentConfig.CoreDatabase, "socket", None)
    connectUrl = getattr(wmAgentConfig.CoreDatabase, "connectUrl", None)
    (dialect, junk) = connectUrl.split(":", 1)

    myWMInit = WMInit()
    myWMInit.setDatabaseConnection(dbConfig = connectUrl, dialect = dialect,
                                   socketLoc = socketLoc)
    return

sql = """SELECT blockname, lfn FROM dbsbuffer_block
           INNER JOIN dbsbuffer_file ON
             dbsbuffer_block.id = dbsbuffer_file.block_id
         WHERE dbsbuffer_file.status = 'InPHEDEx'"""

connectToDB()
myThread = threading.currentThread()

results = []
for result in myThread.dbi.processData(sql):
    results.extend(result.fetchall())

blocks = {}
for row in results:
    if row[0] not in blocks:
        blocks[row[0]] = []

    blocks[row[0]].append(row[1])

args = {}
args["url"] = "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader"
args["version"] = "DBS_2_0_9"
args["mode"] = "GET"
dbsApi = DbsApi(args)

badBlocks = []
badFiles = []
for blockName in blocks:
    print("%s:" % blockName)
    blockFiles = []
    try:
        dbsFiles = dbsApi.listFileArray(blockName = blockName)
    except Exception as ex:
        dbsFiles = []

    for dbsFile in dbsFiles:
        blockFiles.append(dbsFile["LogicalFileName"])

    if len(blockFiles) != len(blocks[blockName]):
        print("\tFile count mismatch: %s local, %s global" % (len(blocks[blockName]), len(blockFiles)))
        if blockName not in badBlocks:
            badBlocks.append(blockName)

    for blockFile in blocks[blockName]:
        if blockFile not in blockFiles:
            badFiles.append(blockFile)
            #print "\t File missing: %s" % blockFile
            if blockName not in badBlocks:
                badBlocks.append(blockName)

    #sys.exit(0)

for badFile in badFiles:
    try:
        dbsFile = dbsApi.listFileArray(patternLFN = badFile)[0]
        if dbsFile["Block"]["Name"] not in badBlocks:
            badBlocks.append(dbsFile["Block"]["Name"])
    except Exception as ex:
        continue

print("Bad:")
for blockName in badBlocks:
    print(blockName)




#!/usr/bin/env python
from __future__ import (division, print_function)
from future.utils import viewitems, viewvalues

import time
from collections import defaultdict

from WMCore.REST.CherryPyPeriodicTask import CherryPyPeriodicTask
from WMCore.Services.DBS.DBS3Reader import DBS3Reader
from WMCore.Services.RequestDB.RequestDBWriter import RequestDBWriter


def getChildDatasetsForStepChainMissingParent(reqmgrDB, status):
    """
    :param status: workflow status
    :return: set of child dataset
    """
    results = reqmgrDB.getStepChainDatasetParentageByStatus(status)

    requestsByChildDataset = defaultdict(set)

    for reqName, info in viewitems(results):

        for dsInfo in viewvalues(info):
            for childDS in dsInfo["ChildDsets"]:
                requestsByChildDataset[childDS].add(reqName)
    return requestsByChildDataset


class StepChainParentageFixTask(CherryPyPeriodicTask):
    """
    Upldates StepChain parentage periodically
    """

    def __init__(self, rest, config):

        super(StepChainParentageFixTask, self).__init__(config)
        self.reqmgrDB = RequestDBWriter(config.reqmgrdb_url)
        self.dbsSvc = DBS3Reader(config.dbs_url, logger=self.logger)
        self.statusToCheck = ["closed-out", "announced"]

    def setConcurrentTasks(self, config):
        """
        sets the list of functions which
        """
        self.concurrentTasks = [{'func': self.fixStepChainParentage, 'duration': config.parentageFixDuration}]

    def fixStepChainParentage(self, config):
        """
        Look through the stepchain workflows with ParentageResolved flag is False.
        Fix the StepChain parentage and update the ParentageResolved flag to True
        """
        self.logger.info("Running fixStepChainParentage thread for statuses: %s", self.statusToCheck)
        childDatasets = set()
        requests = set()
        requestsByChildDataset = {}
        for status in self.statusToCheck:
            reqByChildDS = getChildDatasetsForStepChainMissingParent(self.reqmgrDB, status)
            self.logger.info("Retrieved %d datasets to fix parentage, in status: %s",
                             len(reqByChildDS), status)
            childDatasets = childDatasets.union(set(reqByChildDS.keys()))
            # We need to just get one of the StepChain workflow if multiple workflow contains the same datasets. (i.e. ACDC)
            requestsByChildDataset.update(reqByChildDS)

            for wfs in viewvalues(reqByChildDS):
                requests = requests.union(wfs)

        failedRequests = set()
        totalChildDS = len(childDatasets)
        fixCount = 0
        for childDS in childDatasets:
            self.logger.info("Resolving parentage for dataset: %s", childDS)
            start = time.time()
            try:
                failedBlocks = self.dbsSvc.fixMissingParentageDatasets(childDS, insertFlag=True)
            except Exception as exc:
                self.logger.exception("Failed to resolve parentage data for dataset: %s. Error: %s",
                                      childDS, str(exc))
                failedRequests = failedRequests.union(requestsByChildDataset[childDS])
            else:
                if failedBlocks:
                    self.logger.warning("These blocks failed to be resolved and will be retried later: %s",
                                        failedBlocks)
                    failedRequests = failedRequests.union(requestsByChildDataset[childDS])
                else:
                    fixCount += 1
                    self.logger.info("Parentage for '%s' successfully updated. Processed %s out of %s datasets.",
                                     childDS, fixCount, totalChildDS)
            timeTaken = time.time() - start
            self.logger.info("    spent %s secs on this dataset: %s", timeTaken, childDS)

        requestsToUpdate = requests - failedRequests

        for request in requestsToUpdate:
            try:
                self.reqmgrDB.updateRequestProperty(request, {"ParentageResolved": True})
                self.logger.info("Marked ParentageResolved=True for request: %s", request)
            except Exception as exc:
                self.logger.error("Failed to update 'ParentageResolved' flag to True for request: %s", request)

        msg = "A total of %d requests have been processed, where %d will have to be retried in the next cycle."
        self.logger.info(msg, len(requestsToUpdate), len(failedRequests))



from future import standard_library
standard_library.install_aliases()

import logging
import os
import unittest
from http.client import HTTPException

from WMCore.Services.Requests import Requests
from WMCore.WMBase import getTestBase
from WMQuality.WebTools.RESTBaseUnitTest import RESTBaseUnitTest
from WMQuality.WebTools.RESTServerSetup import DefaultConfig


class PyCurlRESTServer(RESTBaseUnitTest):
    """
    Loads the rest server and test the upload
    """

    def initialize(self):
        self.config = DefaultConfig('WMCore_t.Services_t.PyCurlRESTModel')
        self.config.Webtools.environment = 'development'
        self.config.Webtools.error_log_level = logging.ERROR
        self.config.Webtools.access_log_level = logging.ERROR
        self.config.Webtools.port = 8888
        self.config.Webtools.host = '127.0.0.1'
        self.config.UnitTests.object = 'WMCore_t.Services_t.PyCurlRESTModel'
        self.requestHandler = Requests('http://127.0.0.1:8888/unittests/rest/')

    def testFileUpload(self):
        """
        The method upload a file (data/TestUpload.txt) and check if the server API has saved it
        """
        uploadedFilename = 'UploadedFile.txt'
        fileName = os.path.join(getTestBase(), 'WMCore_t/Services_t/TestUpload.txt')
        # Check the uploaded file is not there
        if os.path.isfile(uploadedFilename):
            os.remove(uploadedFilename)
            self.assertFalse(os.path.isfile(uploadedFilename))
        # do the upload
        res = self.requestHandler.uploadFile(fileName, 'http://127.0.0.1:8888/unittests/rest/file/')
        # the file is there now (?)
        self.assertTrue(os.path.isfile(uploadedFilename))
        self.assertEqual(open(uploadedFilename).read(), open(fileName).read())
        # delete the uploaded file
        os.remove(uploadedFilename)
        self.assertTrue(b'Success' in res)

    def testFailingFileUpload(self):
        """
        The method upload a file (data/TestUpload.txt) and check if the server API has saved it
        """
        uploadedFilename = 'UploadedFile.txt'
        fileName = os.path.join(getTestBase(), 'WMCore_t/Services_t/TestUpload.txt')
        # Check the uploaded file is not there
        if os.path.isfile(uploadedFilename):
            os.remove(uploadedFilename)
            self.assertFalse(os.path.isfile(uploadedFilename))
        # do the upload using the wrong address
        try:
            res = self.requestHandler.uploadFile(fileName, 'http://127.0.0.1:8888/unittests/rest/iAmNotThere/')
            self.fail()
        except HTTPException as he:
            self.assertEqual(he.status, 404)
        self.assertFalse(os.path.isfile(uploadedFilename))


if __name__ == "__main__":
    unittest.main()
